# -*- coding: utf-8 -*-
#$Id: Install.py 132 2006-04-19 14:16:38Z taka $
#############################################################################
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

from Products.CMFCore.utils import getToolByName
from Products.CMFFormController.FormAction import FormActionKey
from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin
from Products.Archetypes.public import listTypes

from StringIO import StringIO

from Products.CMFDynamicViewFTI.migrate import migrateFTIs

from Products.ATCTSmallSample.config import *

# この関数はPloneUI上の「プロダクトを追加・削除」でインストールを実行したと
# きに呼び出されます。ここでスキン・CSS・JavaScriptの登録／コンテンツタイプ
# のDynamicViewFTIへの変換・PortalFacotryへの登録などを行います。
def install(self):
    """Install ATCTSmallSample"""

    # install関数の返値がインストールログになるので、標準出力のかわりに
    # StringIOの一時バッファに出力して、あとでまとめて取り出します。
    out = StringIO()

    # 最初にインストール開始ログを出力します。
    print >> out, "Installing %s" % PROJECTNAME

    # コンテンツタイプを登録して、Ploneで使用できるようにします。listTypes
    # 関数は、registerType()で登録されているクラスオブジェクトの一覧を返し
    # ます。ここで登録した情報はplone/portal_typesで確認・変更することが出
    # 来ます。また、global_allowやdefault_viewなどのパラメータはこの時点で
    # plone/portal_typesに保存されるため、これらのパラメータをプロダクトコ
    # ード上で変更した場合は、設定を更新するためにプロダクトを再追加する必
    # 要があります。
    classes = listTypes(PROJECTNAME)
    installTypes(self, out,
                 classes,
                 PROJECTNAME)
    print >> out, "Installed types"


    # コンテンツタイプをDynamicViewFTIに変換します。この変換を行うと、ペー
    # ジの表示方法をPloneUI上で指定することが出来るようになります。Plone標
    # 準ではフォルダのViewに写真一覧などがありますが、新規に作成したコンテ
    # ンツタイプでは独自に複数のViewを用意することも出来ます。指定方法はコ
    # ンテンツタイプ実装クラスのsuppl_viewsを参照してください。
    migrated = migrateFTIs(self, product=PROJECTNAME)
    print >>out, "Switched to DynamicViewFTI: %s" % ', '.join(migrated)

    # コンテンツタイプのportal_factoryを有効にします。
    # portal_factoryを有効にすると、コンテンツのインスタンスを新規に作成し
    # たときに、インスタンスはテンポラリ領域に作成されます。保存ボタンを押
    # すとインスタンスが実際のフォルダに移動されます。この仕組みを使うと、
    # 作成途中で放置されたインスタンスの残骸が一般に公開されているフォルダ
    # に残りません。
    factory = getToolByName(self, 'portal_factory')
    types = factory.getFactoryTypes().keys()
    for add_type in ('SmallContent',):
        if add_type not in types:
            types.append(add_type)
            factory.manage_setPortalFactoryTypes(listOfTypeIds = types)

    print >> out, "Added %s to portal_factory" % PROJECTNAME

    # リソース(cssやJavaScriptなど)をインストールします。プロダクト毎に構成
    # が異なるため、別関数にしました。
    install_resources(self, out)

    # インストール実行ログを返します。
    return out.getvalue()


# リソース(cssやJavaScriptなど)をインストールします。
def install_resources(self, out):
    # Install skin
    # portal_skinsのスキンセレクションにATCTSmallSampleのレイヤーを追加しま
    # す。レイヤーの追加位置は'custom'の次になります。
    install_subskin(self, out, GLOBALS)
    print >> out, "Installed skin"

    # plone/portal_css インスタンスを取得します。
    portal_css = getToolByName(self, 'portal_css')

    # portal_css にCSSを登録します。expressionにTAL式を記述できるので、利用
    # 条件でportal_typeのチェックを行ったり、複雑なチェックが必要な場合もチ
    # ェック用Pythonスクリプトを用意しておいて呼び出すなど、Ploneの仕組み上
    # で柔軟な処理を行うことが出来ます。
    # portal_cssに登録したCSSは、プロダクトのアンインストール時に自動的に削
    # 除されるため、uninstall用のコードを書く必要はありません(Plone-2.1.2の
    # 場合)。
    portal_css.manage_addStylesheet(# 登録するCSSのオブジェクト名
                                    id = 'smallcontent.css',
                                    # TAL式でCSSの利用条件を指定
                                    expression = '',
                                    # ブラウザのメディアタイプを指定
                                    media = 'all',
                                    # portal_cssに登録されるタイトル
                                    title = 'inline SmallContent styles',
                                    # 登録時の有効・無効設定
                                    enabled = True)

    print >> out, "Installed css"


# PloneUI上の「プロダクトを追加・削除」でアンインストールを実行したときに呼
# び出されます。ここで登録解除が必要な処理を行います。
def uninstall(self):
    # install時と同様にアンインストールログの出力先バッファを作成します。
    out = StringIO()

    # リソースのアンインストール処理を呼び出します。
    uninstall_resources(self, out)

    print >> out, "Successfully uninstalled %s." % PROJECTNAME
    return out.getvalue()


# リソースのアンインストール処理を行います。
def uninstall_resources(self, out):
    """Remove the js and css files from the resource registries"""

    # スキンのアンインストール処理を呼び出します。
    uninstallSkins(self, out)

    print >>out, "Resource files removed"


# スキンのアンインストール処理を行います。
def uninstallSkins(self, out):
    # plone/portal_skins インスタンスを取得します。
    skins_tool = getToolByName(self, 'portal_skins')

    # スキンセレクションの一覧を取得します。標準では'Plone Default'と
    # 'Plone Tableless'が取得されます。
    skins = skins_tool.getSkinSelections()

    # 各スキンセレクション毎にATCTSmallSampleレイヤーの削除処理を行います。
    for skin in skins:
        # スキンセレクションのレイヤー一覧を取得します。
        path = skins_tool.getSkinPath(skin)

        # レイヤー一覧はカンマ区切りのstring型なので、カンマで分割して前後
        # の余分な空白を除去します。
        path = [p.strip() for p in path.split(',') if p]

        # pathのリスト内にPROJECTNAMEと一致する要素がある間、その要素を取り
        # 除く処理を繰り返します。
        while PROJECTNAME in path:
            # リスト内から要素を取り除きます。ここではPROJECTNAMEを取り除く
            # ので、skinsディレクトリ以下にはconfig.PROJECTNAMEと同じ名前の
            # ディレクトリがあることを想定しています。このため、複数のスキ
            # ンディレクトリがskins以下にある場合はこの処理では取り除かれま
            # せん。
            path.remove(PROJECTNAME)

        # ATCTSmallSampleのスキンレイヤーを取り除いたレイヤー一覧をカンマ区
        # 切りの文字列に連結して保存します。
        skins_tool.addSkinSelection(skin, ','.join(path))

    print >> out, "Uninstalled skin"

